%% LifeTimeTable for corresponding Decision Points
%  This code modifies the Lifetime Table based on the current number of decision points
%  With respect to this, Lifetime Table consists of the primary and secondary lifetimes corresponding to 
%  the available options (which is determined by the number of decision points being analyzed)

FieldLifeTime = 50;
Year = 10; %Change this value to define the desired number of decision points

% 1. Preallocation of Essential Parameters

if Year == 2
    N_DecisionPoint = 25;
    LT_DecisionPoint = 0:2:FieldLifeTime;
elseif Year == 4
    N_DecisionPoint = 12;
    LT_DecisionPoint = 0:4:FieldLifeTime;
elseif Year == 6
    N_DecisionPoint = 8;
    LT_DecisionPoint = 0:6:FieldLifeTime;
elseif Year == 8
    N_DecisionPoint = 6;
    LT_DecisionPoint = 0:8:FieldLifeTime;    
elseif Year == 10
    N_DecisionPoint = 5;
    LT_DecisionPoint = 0:10:FieldLifeTime;
end

N_Phase1n2LifeTime_DiffDP = sum(0:1:N_DecisionPoint+1);
LifeTime_Table_DiffDP = zeros(N_Phase1n2LifeTime_DiffDP,2); 
meanNPVvector_DiffDP = zeros(N_Phase1n2LifeTime_DiffDP,3);

% 2. Defining LifeTime Phase 1
NPV_element_vector_LT1 = zeros(N_DecisionPoint+1,N_DecisionPoint+1);

for i_DecisionPoint = 1:N_DecisionPoint
    for k_DecisionPoint = 1:N_DecisionPoint+1
        NPV_element_vector_LT1(k_DecisionPoint,i_DecisionPoint) = LT_DecisionPoint(1,k_DecisionPoint);
        if k_DecisionPoint > 1
            NPV_element_vector_LT1(k_DecisionPoint,N_DecisionPoint-k_DecisionPoint+3:N_DecisionPoint+1) = LT_DecisionPoint(1,N_DecisionPoint+1)+1;
        else
            NPV_element_vector_LT1(k_DecisionPoint,i_DecisionPoint+1) = LT_DecisionPoint(1,k_DecisionPoint); 
        end
    end
end

[row_NPV_LT1_element,col_NPV_LT1_element]=find(NPV_element_vector_LT1(:,:) <= LT_DecisionPoint(1,N_DecisionPoint+1));

for k = 1: N_Phase1n2LifeTime_DiffDP
    meanNPVvector_DiffDP(k,1) = NPV_element_vector_LT1(col_NPV_LT1_element(k),row_NPV_LT1_element(k));
end

% 3. Defining LifeTime Phase 2
NPV_element_vector_LT2 = zeros(N_DecisionPoint+1,N_DecisionPoint+1);

for i_DecisionPoint = 1:N_DecisionPoint
    for k_DecisionPoint = 1:N_DecisionPoint+1
        NPV_element_vector_LT2(k_DecisionPoint,i_DecisionPoint) = LT_DecisionPoint(1,i_DecisionPoint);
        if k_DecisionPoint > 1
            NPV_element_vector_LT2(k_DecisionPoint,N_DecisionPoint-k_DecisionPoint+3:N_DecisionPoint+1) = LT_DecisionPoint(1,N_DecisionPoint+1)+1;
        else
            NPV_element_vector_LT2(k_DecisionPoint,i_DecisionPoint+1) = LT_DecisionPoint(1,i_DecisionPoint+1); 
        end
    end
end

[row_NPV_LT2_element,col_NPV_LT2_element]=find(NPV_element_vector_LT2(:,:) < LT_DecisionPoint(1,N_DecisionPoint+1));

for k = 1: N_Phase1n2LifeTime_DiffDP-1
    meanNPVvector_DiffDP(k,2) = NPV_element_vector_LT2(col_NPV_LT2_element(k),row_NPV_LT2_element(k));
end

% 4. Defining LifeTime Table

LifeTime_Table_DiffDP(:,1) = meanNPVvector_DiffDP(:,1);
LifeTime_Table_DiffDP(:,2) = meanNPVvector_DiffDP(:,2);


