%% SRDM Approach (Economic Uncertainties) 
%  This code determines the optimal decision policy using SRDM approach by including economic uncertainties
%  It consists of the modified version of Sections 5 and 6 of Modified LSM Algorithm.m

%  1. Path Table Generation (Modified LSM algorithm)
%  This section determines the optimal stopping time given the switch time at every year
%  Regression Analysis is applied
%  The NPV corresponding to the optimal stopping time is recorded into the PathTable

N_MC = size(ObsRate_Phase1n2_real_time_ShiftTime,1);
FieldLifeTime = size(ObsRate_Phase1n2_real_time_ShiftTime,2);
SRDM_Shift_Stop_OptNPV_real = zeros(N_MC,1);
        
PathTable = zeros(N_MC,FieldLifeTime+1);
PathTable(:,end) = NPVtable_LTPhase1_LTPhase2_real(FieldLifeTime+1,1,:);
Phase2_StopTime_real_Phase1LifeTime = zeros(N_MC,FieldLifeTime+1);

for k_Shift_time = 1: FieldLifeTime
        NPV_2Phases_matrix = zeros(N_MC,k_Shift_time+1); 
    for k_Stop_time = 1: k_Shift_time+1          
        NPV_2Phases_matrix(:,k_Stop_time) = NPVtable_LTPhase1_LTPhase2_real(FieldLifeTime-k_Shift_time+1,k_Stop_time,:);
    end
    for k_StopTime = 1: k_Shift_time    
        if k_StopTime == 1 
            x_data_stop_SRDM = ObsRate_Phase1n2_real_time_ShiftTime(:,1:FieldLifeTime-k_StopTime,FieldLifeTime-k_Shift_time+1); 
            oilprice_data = OilPrice_Matrix(:,FieldLifeTime-k_StopTime);
            opex_data = Opex_Phase2_Matrix(:,FieldLifeTime-k_StopTime);
            X_stop_SRDM = [ones(N_MC,1) x_data_stop_SRDM oilprice_data opex_data]; 
            y_data_stop_matrix = [NPV_2Phases_matrix(:,k_Shift_time-k_StopTime+1) NPV_2Phases_matrix(:,k_Shift_time-k_StopTime+2)];
            [Stop_value,Stop_idx] = max([X_stop_SRDM*regress(NPV_2Phases_matrix(:,k_Shift_time-k_StopTime+1),X_stop_SRDM) X_stop_SRDM*regress(NPV_2Phases_matrix(:,k_Shift_time-k_StopTime+2),X_stop_SRDM)],[],2);
            for i_MC = 1: N_MC
                SRDM_Shift_Stop_OptNPV_real(i_MC,1) = y_data_stop_matrix(i_MC,Stop_idx(i_MC));
            end    
        else
            x_data_stop_SRDM = ObsRate_Phase1n2_real_time_ShiftTime(:,1:FieldLifeTime-k_StopTime,FieldLifeTime-k_Shift_time+1); 
            if FieldLifeTime-k_StopTime == 0 
                oilprice_data = OilPrice_Matrix(:,1:FieldLifeTime-k_StopTime);
                opex_data = Opex_Phase2_Matrix(:,1:FieldLifeTime-k_StopTime);
            else
                oilprice_data = OilPrice_Matrix(:,FieldLifeTime-k_StopTime);  
                opex_data = Opex_Phase2_Matrix(:,FieldLifeTime-k_StopTime);
            end
            X_stop_SRDM = [ones(N_MC,1) x_data_stop_SRDM oilprice_data opex_data]; 
            y_data_stop_matrix = [NPV_2Phases_matrix(:,k_Shift_time-k_StopTime+1) SRDM_Shift_Stop_OptNPV_real(:,1)];
            [Stop_value,Stop_idx] = max([X_stop_SRDM*regress(NPV_2Phases_matrix(:,k_Shift_time-k_StopTime+1),X_stop_SRDM) X_stop_SRDM*regress(SRDM_Shift_Stop_OptNPV_real(:,1),X_stop_SRDM)],[],2);
            for i_MC = 1: N_MC
                SRDM_Shift_Stop_OptNPV_real(i_MC,1) = y_data_stop_matrix(i_MC,Stop_idx(i_MC));
            end  
        end
    end     
    for i_MC = 1: N_MC
        PathTable(i_MC,FieldLifeTime-k_Shift_time+1) = SRDM_Shift_Stop_OptNPV_real(i_MC,1);
        for k_Stop_time = 1: k_Shift_time+1
            if SRDM_Shift_Stop_OptNPV_real(i_MC,1) == NPV_2Phases_matrix(i_MC,k_Stop_time)
                Phase2_StopTime_real_Phase1LifeTime(i_MC,FieldLifeTime-k_Shift_time+1) = k_Stop_time-1;
            end
        end
    end  
end

% 2. SRDM Approach (Modified LSM algorithm)
% This section determines the optimal decision policy based on SRDM approach

FieldLifeTime = size(ObsRate_Phase1n2_real_time_ShiftTime,2);
y_data_continuation = zeros(N_MC,1);
y_data_shift = zeros(N_MC,1);
y_reg_continuation = zeros(N_MC,1);
y_reg_shift = zeros(N_MC,1);
x_data = ObsRate_Phase1n2_real_time_ShiftTime(:,:,FieldLifeTime+1);

ValueTable = zeros(N_MC,FieldLifeTime+1);
ValueTable(:,end) = PathTable(:,end);
SRDM_DecisionTable = zeros(N_MC,FieldLifeTime+1);
SRDM_DecisionTable(:,end) = 1;
oilprice_data = OilPrice_Matrix(:,:);
opex_phase1_data = Opex_Phase1_Matrix(:,:);
opex_phase2_data = Opex_Phase2_Matrix(:,:);
capex_phase2after1_data = Capex_Phase2After1_Matrix(:,:);

for k_Shift_time = 1:FieldLifeTime
    if FieldLifeTime-k_Shift_time == 0
        X_shift = [ones(N_MC,1) x_data(:,1:FieldLifeTime-k_Shift_time) oilprice_data(:,1:FieldLifeTime-k_Shift_time) opex_phase1_data(:,1:FieldLifeTime-k_Shift_time) opex_phase2_data(:,1:FieldLifeTime-k_Shift_time) capex_phase2after1_data(:,1:FieldLifeTime-k_Shift_time)];
        X_continuation = [ones(N_MC,1) x_data(:,1:FieldLifeTime-k_Shift_time) oilprice_data(:,1:FieldLifeTime-k_Shift_time) opex_phase1_data(:,1:FieldLifeTime-k_Shift_time) opex_phase2_data(:,1:FieldLifeTime-k_Shift_time) capex_phase2after1_data(:,1:FieldLifeTime-k_Shift_time)];
    else
        X_shift = [ones(N_MC,1) x_data(:,1:FieldLifeTime-k_Shift_time) oilprice_data(:,FieldLifeTime-k_Shift_time) opex_phase1_data(:,FieldLifeTime-k_Shift_time) opex_phase2_data(:,FieldLifeTime-k_Shift_time) capex_phase2after1_data(:,FieldLifeTime-k_Shift_time)]; 
        X_continuation = [ones(N_MC,1) x_data(:,1:FieldLifeTime-k_Shift_time) oilprice_data(:,FieldLifeTime-k_Shift_time) opex_phase1_data(:,FieldLifeTime-k_Shift_time) opex_phase2_data(:,FieldLifeTime-k_Shift_time) capex_phase2after1_data(:,FieldLifeTime-k_Shift_time)];
    end
    y_data_shift(:) = PathTable(:,FieldLifeTime-k_Shift_time+1); 
    coef_shift = regress(y_data_shift,X_shift);
    y_reg_shift(:) = X_shift*coef_shift;
    y_data_continuation(:) = ValueTable(:,FieldLifeTime-k_Shift_time+2);
    coef_continuation = regress(y_data_continuation,X_continuation);
    y_reg_continuation(:) = X_continuation*coef_continuation;
    ENPVTable_Comparison = [y_reg_shift, y_reg_continuation];
    NPV_SRDM = [y_data_shift, y_data_continuation];
    [ENPV_SRDM,ENPV_SRDM_idx] = max(ENPVTable_Comparison,[],2);
    
    for i_MC = 1: N_MC
        ValueTable(i_MC,FieldLifeTime-k_Shift_time+1) = NPV_SRDM(i_MC,ENPV_SRDM_idx(i_MC));
        if y_reg_continuation(i_MC) > y_reg_shift(i_MC)
            SRDM_DecisionTable(i_MC,FieldLifeTime-k_Shift_time+1) = 0;
        else
            SRDM_DecisionTable(i_MC,FieldLifeTime-k_Shift_time+1) = 1;
        end
    end
end

ShiftValue = PathTable(:,1);
ExpShiftValue = mean(ShiftValue);

ContinuationValue = ValueTable(:,1);
ExpContinuationValue = mean(ContinuationValue);
EVWII = ExpContinuationValue;
VOI = EVWII - EVWOI;
VOI_over_EVWOI_pct = VOI/EVWOI*100;

[~,DWI_Phase1LifeTime_idx] = max(SRDM_DecisionTable,[],2);
DWI_Phase1LifeTime = DWI_Phase1LifeTime_idx-1;

DWI_Phase2LifeTime = zeros(N_MC,1);
for i_MC = 1:N_MC
    Phase1LifeTime_idx_real = DWI_Phase1LifeTime_idx(i_MC,1);
    DWI_Phase2LifeTime(i_MC,1) = Phase2_StopTime_real_Phase1LifeTime(i_MC,Phase1LifeTime_idx_real);
end

DWI_LifeTime = DWI_Phase1LifeTime+DWI_Phase2LifeTime;
