%% Data Generation (only appliable to the Reservoir Model described in the work) 
%  This code generates the input data used for the modified LSM method from
%  the output file of the Reservoir Simulation
%  The RSM file only provides the cumulative production (cubic meter) for each year
%  Since the state variable is measured oil rates (MMbbl/year), calculation
%  of oil rate and unit conversion are done by this code

Year = 5;
N_realization = 100;

% DataPoint defines the number of rows corresponding to the data at particular year
% For instance, the data at Year 1 is in row 9
% DataPoint is very case-dependent and needs to be modified
% accordingly if different reservoir model is used. 
DataPoint = [9;10;11;12;13]; 
Data_Matrix = zeros(N_realization,Year,Year+1);
Foldername = 'C:\Users\cuthb\OneDrive - NTNU\2D_LSM_Reservoir_Model';

for i_Year = 1: Year+1 
    for i_realization = 1: N_realization 
        Filename = [Foldername,'\Switch_at_Year_',num2str(i_Year),'\TextFile\2D_MODEL_SWITCH_Y',num2str(i_Year),'_R',num2str(i_realization),'.txt'];
        DataFile = importdata(Filename);
        
        % Columns 4 and 5 correspond the cumulative production for top and bottom layer
        Y1_Data_metric = (DataFile.data(DataPoint(1),4) + DataFile.data(DataPoint(1),5))/1000000;
        Y2_Data_metric = ((DataFile.data(DataPoint(2),4) - DataFile.data(DataPoint(1),4)) + (DataFile.data(DataPoint(2),5) - DataFile.data(DataPoint(1),5)))/1000000;
        Y3_Data_metric = ((DataFile.data(DataPoint(3),4) - DataFile.data(DataPoint(2),4)) + (DataFile.data(DataPoint(3),5) - DataFile.data(DataPoint(2),5)))/1000000;
        Y4_Data_metric = ((DataFile.data(DataPoint(4),4) - DataFile.data(DataPoint(3),4)) + (DataFile.data(DataPoint(4),5) - DataFile.data(DataPoint(3),5)))/1000000;
        Y5_Data_metric = ((DataFile.data(DataPoint(5),4) - DataFile.data(DataPoint(4),4)) + (DataFile.data(DataPoint(5),5) - DataFile.data(DataPoint(4),5)))/1000000;
        
        % Unit conversion from cubic meter to bbl
        Y1_Data = Y1_Data_metric.*6.29;
        Y2_Data = Y2_Data_metric.*6.29;
        Y3_Data = Y3_Data_metric.*6.29;
        Y4_Data = Y4_Data_metric.*6.29;
        Y5_Data = Y5_Data_metric.*6.29;
        
        Data_Matrix(i_realization,1,i_Year) = Y1_Data;
        Data_Matrix(i_realization,2,i_Year) = Y2_Data;
        Data_Matrix(i_realization,3,i_Year) = Y3_Data;
        Data_Matrix(i_realization,4,i_Year) = Y4_Data;
        Data_Matrix(i_realization,5,i_Year) = Y5_Data;
    end
end
